package JIP;

import java.applet.Applet;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import JIP.engine.*;
import Ugosoft.awt.*;
import Ugosoft.io.*;

public class JIPConsole extends Applet implements JIPEventListener, JIPErrorListener, ActionListener, ItemListener
{   
    private static final String PROMPT  = "JIP:-";

	// Prolog Engine
	private JIPEngine 		m_prolog;
	
	// Query Handle
	private int 			m_nQueryHandle;

	// Dialog for Tracing
    private TraceDialog 	m_traceDlg;
    
    // Panel's Field
    private Button           m_btnNewProg;
    private Button           m_btnOpenProg;
    private Button           m_btnReset;
	private Button           m_btnClose;
	private TerminalTextArea m_mainArea;
    private Checkbox         m_chkTrace;

	// Stream
	private PrintStream 	m_outs 			= null;

	// Variable
	private static String 	m_strBasePath;
        
	// To run JIP as an application
	public static void main(String args[])
    {
        Runtime.getRuntime().runFinalizersOnExit(true);

        m_strBasePath = "file:/" + System.getProperties().getProperty("user.dir") + "/";

    	// Main Frame 
        ApplicationFrame frame = new ApplicationFrame("JIP - Java Internet Prolog");        
        Dimension ds = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setSize((int) ((double)ds.width * (7.0/10.0)), (int)((double)ds.height * (8.0/10.0)));
        // icon
        frame.setIconImage(Toolkit.getDefaultToolkit().getImage(frame.getClass().getResource("/icoJip.gif")));

        // The applet
        JIPConsole app = new JIPConsole();
        app.init();
        
        frame.add(app);
        
        app.setMainFrame(frame);        
        
        frame.show();                                   

        app.printHeader();
    }
	
	// Constructor
	public JIPConsole()
	{
		super();
	}
	
	// Initializzation
    public void init()
    {
   		setBackground(Color.gray.brighter());
    	setLayout(new BorderLayout());
        
    	// Environment variables
    	try
    	{    		
    		m_strBasePath = getCodeBase().toString();
    	}    
        catch(NullPointerException ex){}

        Panel topPanel = new Panel();
        topPanel.setLayout(new GridLayout(2,1));
        Label lb = new Label("JIP - Java Internet Prolog v" + JIPEngine.getVersion(), Label.CENTER);
        lb.setFont(new Font("TimesRoman", Font.BOLD, 16));
        lb.setForeground(Color.white);
        topPanel.add(lb);
        lb = new Label("by Ugo Chirico - http://www.geocities.com/jiprolog", Label.CENTER);
        lb.setFont(new Font("TimesRoman", Font.BOLD, 12));
        lb.setForeground(Color.white);
        topPanel.add(lb);

        topPanel.setBackground(Color.cyan.darker());
        add("North", topPanel);
        
        // Main Panel 
        Panel mainPanel = new Panel();
        mainPanel.setLayout(new GridLayout(1,1));
        
        m_chkTrace    = new Checkbox("Trace", false);
        m_btnNewProg  = new Button("New");
        m_btnOpenProg = new Button("Open");        
        m_btnReset    = new Button("Reset");
		m_btnClose    = new Button("Stop");
		Label copyr	  = new Label("Copyright, Ugo Chirico", Label.CENTER);
		
		m_mainArea 	  = new TerminalTextArea("", 0, 0, TextArea.SCROLLBARS_NONE);//SCROLLBARS_VERTICAL_ONLY);

		copyr.setFont(new Font("Arial", Font.BOLD, 9));
		copyr.setForeground(Color.white);
		
        m_chkTrace.setForeground(Color.white);
        m_chkTrace.setFont(new Font("Arial", Font.BOLD, 12));
		m_mainArea.setFont(new Font("Courier", Font.PLAIN, 12));
		               
        m_btnNewProg.addActionListener(this);
        m_btnReset.addActionListener(this);
        m_btnOpenProg.addActionListener(this);
        m_btnClose.addActionListener(this);
        m_chkTrace.addItemListener(this);
        m_mainArea.addActionListener(this);
        
        m_btnNewProg.setEnabled(true);
        m_btnOpenProg.setEnabled(true);
		m_btnClose.setEnabled(false);

		mainPanel.add(m_mainArea);
        add("Center", mainPanel);
                                       
		Panel panButtons = new Panel();
    
        GridBagLayout layout   = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
    
        panButtons.setLayout(layout);
        
		Panel pan = new Panel();
		pan.setLayout(new GridLayout(1, 4, 3, 0));

		pan.add(m_btnNewProg);
		pan.add(m_btnOpenProg);
		pan.add(m_btnReset);
		pan.add(m_btnClose);		
		
        // Repositioning components on main panel
        gbc.gridwidth = 3;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx = GridBagConstraints.RELATIVE;
        gbc.insets = new Insets(3, 5, 3, 5);    
        gbc.weightx = 1;   
		
		layout.setConstraints(pan, gbc);
        panButtons.add(pan);
        
        gbc.anchor = GridBagConstraints.EAST;
        gbc.insets = new Insets(3, 10, 3, 0);    
        layout.setConstraints(m_chkTrace, gbc);
        panButtons.add(m_chkTrace);
		gbc.fill = GridBagConstraints.NONE;
		layout.setConstraints(copyr, gbc);
        panButtons.add(copyr);				
                                
        panButtons.setBackground(Color.cyan.darker());
        add("South", panButtons);
				
		// Set OutputStream
		m_outs = new TextAreaPrintStream(new TextAreaOutputStream(m_mainArea));
		
        // New instance of prolog engine
        m_prolog = new JIPEngine(null, m_strBasePath, null, m_outs);
        
        // Add EventListener
        m_prolog.addEventListener(this);

        // Add ErrorListener
        m_prolog.addErrorListener(this);
                
        m_mainArea.setPrompt(PROMPT);        
	}

	// Start 
	public void start()
	{
        printHeader();
                
		m_strBasePath = getCodeBase().toString();
		m_prolog.setBasePath(m_strBasePath);
		
	    // Search for main frame 
		// (used for predicates concerning windows such as get/1, wmsgbox, etc.)	
		Container ct = getParent();
		while(! (ct instanceof Frame))
    	{
			ct = ct.getParent();
    	}
    	
    	setMainFrame((Frame)ct);
      
        m_mainArea.requestFocus();              
	}
                 
    public void stop()
    {
		m_prolog.closeAllQueries();
    }
    
	// Main frame
	void setMainFrame(Frame fr)
	{
		m_prolog.setMainFrame(fr);
        m_traceDlg = new TraceDialog(fr);
        m_prolog.addTraceListener(m_traceDlg);
        m_prolog.addEventListener(m_traceDlg);
	}	

	// Consult a prolog file 
	public void consult(String strFilePath)
	{
        waitCursor(true);
        
	  	try
        {
        	URL url = new URL(m_strBasePath + strFilePath);
        	DataInputStream ins  = new DataInputStream(url.openStream());
        
            EditFrame edtFrame = new EditFrame(m_prolog, ins);
            edtFrame.show();
        	ins.close();            
        }
        catch(MalformedURLException ex)
        {
       		System.out.println(ex.toString());
           	return;
        }
        catch(IOException ex)
        {
       		System.out.println(ex.toString());
           	return;
        }
                
        waitCursor(false);
	}
    
    public void actionPerformed(ActionEvent evt)
    {
        if(evt.getSource() == m_mainArea)
        {            
            String strQuery = evt.getActionCommand();
            
            if(m_prolog.moreSolutions(m_nQueryHandle))
            {
                m_outs.println();
                
                if(strQuery.equals("y"))
                {                                      
                    m_chkTrace.setEnabled(false);
                    m_btnReset.setEnabled(false);   
                    m_btnNewProg.setEnabled(false);   
                    m_btnOpenProg.setEnabled(false);   
					m_btnClose.setEnabled(true);
                    
                    if(m_chkTrace.getState())
                    {
                        m_traceDlg.show();
                    }

                    waitCursor(true);
                    m_prolog.nextSolution(m_nQueryHandle);
                }
                else
                {
                    m_traceDlg.hide();

                    m_prolog.closeQuery(m_nQueryHandle);   
                }
            }
            else
            {                                                  
                if(strQuery.equals(""))
                {                    
                    m_mainArea.prompt();     
                    return;                    
                }
                
                m_mainArea.append("\n");                
                                                                              
                JIPTerm query = null;
                try
                {                                     
                    query = JIPTerm.parseQuery(strQuery);
                }
                catch(JIPSyntaxErrorException ex)
                {                    
                    m_mainArea.append(ex.getMessage());
                    m_mainArea.prompt();     
                    return;
                }                    
                
                waitCursor(true);
                   
                m_chkTrace.setEnabled(false);                
                m_mainArea.setEditable(false);
                m_btnReset.setEnabled(false);   
                m_btnNewProg.setEnabled(false);   
                m_btnOpenProg.setEnabled(false);
				m_btnClose.setEnabled(true);

                if(m_chkTrace.getState())
                {
                    m_traceDlg.show();                    
                }

                m_nQueryHandle = m_prolog.openQuery(query);                                     				
            }          
        }
	   	else if(evt.getSource() == m_btnNewProg)    	
	   	{
            EditFrame edtFrame = new EditFrame(m_prolog, false);
            edtFrame.show();
        }  	
        else if(evt.getSource() == m_btnOpenProg)        
        {
            EditFrame edtFrame = new EditFrame(m_prolog, true);
            edtFrame.show();
        }
        else if(evt.getSource() == m_btnReset)        
        {
            m_prolog.reset();
            m_outs.println();
            m_outs.println("Reset OK");
            
            m_mainArea.prompt();     

            m_mainArea.requestFocus();
        }		
        else if(evt.getSource() == m_btnClose)        
        {
			m_prolog.closeQuery(m_nQueryHandle);   
        }
    }

    public void itemStateChanged(ItemEvent e)
    {
        m_prolog.setTrace(m_chkTrace.getState());
        m_mainArea.requestFocus();
    }
    
    
	// Applet Info
	public String getAppletInfo()
	{
        return "JIP - Java Internet Prolog V" + JIPEngine.getVersion() 
			+ " by Ugo Chirico." 
			+ "\nCopyright, Ugo Chirico "
			+ "\nContacts: jiprolog@yahoo.com or http://www.geocities.com/jiprolog";
	}
        
    // A Start event occurred
    public void openNotified(JIPEvent e)
    {
         //System.out.println("Open");
    }
    
    public void moreNotified(JIPEvent e)
    {
         //System.out.println("More");
    }
    
    // A solution event occurred	
    public void solutionNotified(JIPEvent e)
    {         
        // Show Solution 
        m_outs.println("Yes");
        
        JIPTerm term = e.getTerm();
        JIPVariable[] vars = term.getJIPVariables();
        
        for(int i = 0; i < vars.length; i++)
        {
            m_outs.println(vars[i]);
        }                

        waitCursor(false); 
                
        if(!e.getSource().moreSolutions(e.getQueryHandle()))
        {     
            m_prolog.closeQuery(e.getQueryHandle());						
        }
        else
        {
            m_outs.print("more? (y/n):");
			m_btnClose.setEnabled(false);
            m_mainArea.updatePrompt();
        }       

        m_mainArea.setEditable(true);
        m_traceDlg.hide();

        m_mainArea.requestFocus();        
    }
    
    // A Term has been notified with notify/2
    public void termNotified(JIPEvent e)
    {
        m_outs.print("Term notified:\nID = ");
        m_outs.println(e.getID());      
		m_outs.print("Term = ");
        m_outs.println(e.getTerm());      
    }
    
    // The end has been reached because there wasn't more solutions
    public void endNotified(JIPEvent e)
    {
        m_outs.println("No");    
         
        waitCursor(false);
        m_mainArea.requestFocus();
        m_prolog.closeQuery(e.getQueryHandle());
    }
    
    public void closeNotified(JIPEvent e)
    {
		waitCursor(false);
		m_mainArea.prompt();     
		m_chkTrace.setEnabled(true);
        m_btnReset.setEnabled(true);   
        m_btnNewProg.setEnabled(true);   
        m_btnOpenProg.setEnabled(true);   
		m_btnClose.setEnabled(false);
		m_mainArea.setEditable(true);
		
		m_traceDlg.hide();
		
		m_mainArea.requestFocus();
    }

    // An error (exception) has been raised up by prolog engine
    public void errorNotified(JIPErrorEvent e)
    {
        m_outs.println(e.getError());
              
        waitCursor(false);                 
        
        m_mainArea.prompt();        
                
        m_chkTrace.setEnabled(true);
        m_btnReset.setEnabled(true);   
        m_btnNewProg.setEnabled(true);   
        m_btnOpenProg.setEnabled(true);
		m_btnClose.setEnabled(false);
        m_mainArea.setEditable(true);
      
        m_mainArea.requestFocus();          
		
		m_prolog.closeQuery(e.getQueryHandle());
    }
    
    private void waitCursor(boolean bWait)
    {
        if(bWait)
            m_mainArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        else
            m_mainArea.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    
    private void printHeader()
    {
        m_outs.println("*****************************************");
        m_outs.println("* JIP - Java Internet Prolog");
        m_outs.println("* Version: " + JIPEngine.getVersion());
        m_outs.println("* By Ugo Chirico");
        m_outs.println("* http://www.geocities.com/jiprolog");        
        m_outs.println("*****************************************");

        m_mainArea.prompt();        
    }           
}